#!/bin/bash
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://public.eu-egee.org/partners/ for details on 
# the copyright holders.
# For license conditions see the license file or
# http://eu-egee.org/license.html
#
# Authors: 
#      Akos Frohner <Akos.Frohner@cern.ch>
#
# Common configuration script for the gLite Tomcat based
# data management services.

if [ -z "$PACKAGENAME" ]; then
    echo "Error: this script is not supposed to be run directly, but for a service only." >&2
    exit 1
fi

USAGE="$0 [--dryrun] [--withpass] [--vo VO_name] [--values config_values_filename] [--configdir config_directory]"

# the text to be printed at the end
cleanuptext=$cleanuptext"# you can clean up using the following commands\n"

TEMP=$(getopt -o hdwv: --long help,dryrun,withpass,vo:,values:,configdir: -- "$@")
eval set -- "$TEMP"

while true; do
    case "$1" in
        -d|--dryrun)
            DRYRUN='echo Info:'
            shift
            ;;
        -w|--withpass)
            WITHPASS='-p'
            shift
            ;;
        -v|--vo)
            shift
            VO="$1"
            shift
            ;;
        --values)
            shift
            VALUESFILE="$1"
            shift
            ;;
        --configdir)
            shift
            CONFIGDIR="$1"
            shift
            ;;
        -h|--help)
            echo $USAGE
            exit
            ;;
        --)
            # end of options
            shift
            break
            ;;
    esac
done

#################################################################
# default configuration

# RedHat defaults
[ -r /etc/sysconfig/glite ] && source /etc/sysconfig/glite
# Debian defaults
[ -r /etc/default/glite ] && source /etc/default/glite
# node default
[ -r /etc/glite.conf ] && source /etc/glite.conf
# user default
[ -r $HOME/.glite.conf ] && source $HOME/.glite.conf

if [ -z "$GLITE_LOCATION" ]; then
    GLITE_LOCATION='/opt/glite'
    echo "Info: using default install root: $GLITE_LOCATION"
fi
if [ ! -d "$GLITE_LOCATION" ]; then
    echo "Error: '$GLITE_LOCATION' does not exsits." >&2
    echo "Error: Please specify GLITE_LOCATION!" >&2
    echo $USAGE >&2
    exit 1
fi

#################################################################
# configuration file
if [ -z "$VALUESFILE" ]; then
    VALUESFILE="$GLITE_LOCATION/etc/$PACKAGENAME/config.properties"
    echo "Info: using default configuration file: $VALUESFILE"
fi
if [ ! -f "$VALUESFILE" ]; then
    echo "Error: Please specify the configuration values file! (e.g. config.properties)" >&2
    echo $USAGE >&2
    exit 1
fi

#################################################################
# configuration directory
if [ -z "$CONFIGDIR" ]; then
    CONFIGDIR="$GLITE_LOCATION/etc/$PACKAGENAME"
    echo "Info: using default configuration directory: $CONFIGDIR"
fi
if [ ! -d "$CONFIGDIR" ]; then
    echo "Error: Please specify the configuration directory!" >&2
    echo " (e.g. /opt/glite/etc/$PACKAGENAME)" >&2
    echo $USAGE >&2
    exit 1
fi

#################################################################
# Tomcat directory

# system default
[ -r /etc/tomcat5/tomcat5.conf ] && source /etc/tomcat5/tomcat5.conf
# RedHat settings
[ -r /etc/sysconfig/tomcat5 ] && source /etc/sysconfig/tomcat5
# Debian settings
[ -r /etc/default/tomcat5 ] && source /etc/default/tomcat5

# Find Tomcat's webapps dir
if [ ! -d "${CATALINA_BASE}/conf" ]; then
    CATALINA_BASE=${CATALINA_HOME}
fi
if [ ! -d "${CATALINA_BASE}/conf" ]; then
    CATALINA_BASE="/var/tomcat5"
fi
if [ ! -d "${CATALINA_BASE}/conf" ]; then
    CATALINA_BASE="/var/lib/tomcat5"
fi
if [ ! -d "${CATALINA_BASE}/conf" ]; then
    echo "Error: Could not find CATALINA_BASE" >&2
    exit 3
fi
if [ ! -d "$CATALINA_BASE/conf/Catalina/localhost" ]; then
    echo "Error: non default Tomcat installation, where the webapp config directory" >&2
    echo "   does not exists: $CATALINA_BASE/conf/Catalina/localhost" >&2
    exit 2
fi

#################################################################
# testing if the VO name is valid
#   contains only word charaters, '-' and dot
#   not longer than 50 characters
#   not empty (becomes 'default' if not specified)
if [ -z "$VO" ]; then
    echo "Warning: The VO name is empty, using 'default'!" >&2
    VO="default"
fi
invalidchars=$(echo $VO | sed -e 's/[a-zA-Z0-9.-]//g')
if [ -n "$invalidchars" ]; then
    echo "Error: The VO name contains invalid characters: $invalidchars" >&2
    exit 3
fi
overflowchars=$(echo $VO | sed -e 's/^.\{1,50\}//')
if [ -n "$overflowchars" ]; then
    echo "The VO name is more than 50 characters long: $overflowchars" >&2
    exit 3
fi


#################################################################
# parsing the properties file and building a sed script:
#
#  - strip the package name prefix (with and w/o VO name) and the dot
#  - escape slashes
#  - turn the equal sign, surrounded with whitespaces, into slash
#  - prefix the line with "s/"
#  - suffix the line with "/g;"
#
# (custom values can be added by specifying a non-empty sedscript)
#
sedscript=$sedscript" "$(egrep "^$PACKAGENAME\.[^.]+[[:space:]]*=" $VALUESFILE | sed -e "s/^$PACKAGENAME\.//" | sed -e 's/\//\\\//g; s/[[:space:]]*=[[:space:]]*/@\//; s/^/s\/@/; s/$/\/g;/;')
sedscript=$sedscript" "$(egrep "^$PACKAGENAME\.$VO\.[^.]+[[:space:]]*=" $VALUESFILE | sed -e "s/^$PACKAGENAME\.$VO\.//" | sed -e 's/\//\\\//g; s/[[:space:]]*=[[:space:]]*/@\//; s/^/s\/@/; s/$/\/g;/;')
#
# add custom patterns: 
#   @GLITE_LOCATION@ -> $GLITE_LOCATION
#   @HOSTNAME@       -> $(hostname -f)
#   @VO@             -> VO_name from the command line
#
sedscript=${sedscript}" "$(echo "GLITE_LOCATION=$GLITE_LOCATION" | sed -e 's/\//\\\//g; s/[[:space:]]*=[[:space:]]*/@\//; s/^/s\/@/; s/$/\/g;/;')
sedscript=${sedscript}" "$(echo "PACKAGENAME=$PACKAGENAME" | sed -e 's/\//\\\//g; s/[[:space:]]*=[[:space:]]*/@\//; s/^/s\/@/; s/$/\/g;/;')
sedscript=${sedscript}" "$(echo "HOSTNAME="$(hostname -f) | sed -e 's/\//\\\//g; s/[[:space:]]*=[[:space:]]*/@\//; s/^/s\/@/; s/$/\/g;/;')
sedscript=${sedscript}" "$(echo "VO=$VO" | sed -e 's/\//\\\//g; s/[[:space:]]*=[[:space:]]*/@\//; s/^/s\/@/; s/$/\/g;/;')

[ -n "$DRYRUN" ] && $DRYRUN $sedscript

#################################################################
# template instantiation
TEMPLATEDIR="$GLITE_LOCATION/etc/$PACKAGENAME"
if [ ! -d "$TEMPLATEDIR" ]; then
    TEMPLATEDIR="$CONFIGDIR"
    echo "Info: using configuration directory as template directory: $TEMPLATEDIR"
fi

for template in $(ls $TEMPLATEDIR/*.template); do
    if [ ! -f "$template" ]; then
        echo "Error: $template file does not exist!"
        exit 3
    fi
    filename=$CONFIGDIR/${PACKAGENAME}-${VO}-$(basename $template .template)

    if [ -f "$filename" ]; then
        echo "Warning: $filename already exists! Not generated." >&2
    else
        # let be paranoid with passwords
        touch $filename
        chmod 0600 $filename
        # replace the autogenerated patterns
        cat $template | sed -e "$sedscript" >$filename
        cleanuptext=$cleanuptext"    rm $filename\n"
    fi
done

#################################################################
# Tomcat specific configuration
# DB password shall be readable only for tomcat.
#
if [ ! -z "$TOMCAT_USER" ]; then
    $DRYRUN chown $TOMCAT_USER $CONFIGDIR/$PACKAGENAME-*-context.xml
fi

#################################################################
# functions for component specific configuration


function _mysql_create_user {
    $DRYRUN mysql -u root $WITHPASS -e "CREATE DATABASE $DBNAME;
    grant ALL PRIVILEGES on $DBNAME.* to $DBUSER identified by '$DBPASSWORD';
    grant ALL PRIVILEGES on $DBNAME.* to $DBUSER@$HOSTNAME identified by '$DBPASSWORD';
    grant ALL PRIVILEGES on $DBNAME.* to $DBUSER@localhost identified by '$DBPASSWORD';
    flush privileges;"
    cleanuptext=$cleanuptext"      mysql -u root $WITHPASS -e 'DROP DATABASE $DBNAME;'\n"
}

function _oracle_create_user {
    echo "Please ask your Oracle admin to issue the following commands:"
    echo "    CREATE USER $DBUSER IDENTIFIED BY '$DBPASSWORD';"
}

function _mysql_load {
    schema="$1"
    $DRYRUN mysql -u $DBUSER -p${DBPASSWORD} -D $DBNAME -h $(hostname -f) -e "source $CONFIGDIR/$schema;"
}

function _oracle_load {
    schema="$1"
    $DRYRUN echo "sqlplus $DBUSER/$DBPASSWORD@$DBNAME @$CONFIGDIR/$schema"
}

function _print_cleanup {
    unconfigure="${CONFIGDIR}/${PACKAGENAME}-${VO}.unconfigure"
    cleanuptext=$cleanuptext"    rm $unconfigure\n"
    echo -e $cleanuptext
    echo "Info: you can clean up by running '$unconfigure'"
    echo -e "#!/bin/sh\n# this was generated by $0\n" >$unconfigure
    echo -e $cleanuptext >>$unconfigure
    chmod +x $unconfigure
}

function _get_value {
    key="$1"
    value=$(awk -F'[\t =]+' "/^$PACKAGENAME\.$VO\.$key/ { print \$2 }" $VALUESFILE)
    if [ -z "$value" ]; then
        value=$(awk -F'[\t =]+' "/^$PACKAGENAME\.$key/ { print \$2 }" $VALUESFILE)
    fi
}

#################################################################
# database initialization helpers
#
_get_value DBUSER
DBUSER="$value"
_get_value DBNAME
DBNAME="$value"
_get_value DBPASSWORD
DBPASSWORD="$value"

cleanuptext=$cleanuptext"    echo 'Execute the following commands to remove the additional jars:'\n"
# JDBC driver installation
LIBDIR=$GLITE_LOCATION/lib/glite-data-common
for DRIVER in $(cd $LIBDIR; ls *.jar); do
    if [ ! -e "$CATALINA_BASE/common/lib/$DRIVER" ]; then
        $DRYRUN cp $LIBDIR/$DRIVER $CATALINA_BASE/common/lib/$DRIVER
        cleanuptext=$cleanuptext"    echo rm $CATALINA_BASE/common/lib/$DRIVER\n"
    fi
done

#################################################################
# Tomcat configuration
#
CONTEXTFILE="$CATALINA_BASE/conf/Catalina/localhost/${PACKAGENAME}-${VO}-context.xml"
$DRYRUN cp $CONFIGDIR/${PACKAGENAME}-${VO}-context.xml $CONTEXTFILE
if [ ! -z "$TOMCAT_USER" ]; then
    $DRYRUN chown $TOMCAT_USER $CONTEXTFILE
fi
cleanuptext=$cleanuptext"    rm $CONTEXTFILE\n"

